@@ -219,8 +219,6 @@ internal::bfgs_impl(
 
     //ChangedForGPBoost
     //OPTIM_BFGS_TRACE(0, grad_err, rel_sol_change, x_p, d, grad_p, s, y, W);
-    Log::REDebug("GPModel parameter optimization iteration number %d", 0);
-    for (int i = 0; i < std::min((int)x.size(), 5); ++i) { Log::REDebug("(Transformed) parameter[%d]: %g", i, x[i]); }
 
     if (grad_err <= grad_err_tol) {
         init_out_vals = x_p;
@@ -267,19 +265,19 @@ internal::bfgs_impl(
         //ChangedForGPBoost
         //OPTIM_BFGS_TRACE(iter, grad_err, rel_sol_change, x, d, grad, s, y, W);
         if (settings_inp) {
-            settings_inp->opt_iter = iter;
+            settings_inp->opt_iter = iter - 1;
         }
+        //redetermine neighbors for the Vecchia approximation if applicable
+        Vec_t gradient_dummy(2);//"hack" for redermininig neighbors for the Vecchia approximation
+        gradient_dummy[0] = 1.00000000001e30;
+        gradient_dummy[1] = -1.00000000001e30;
+        opt_objfn(x, &gradient_dummy, opt_data);
+        //print trace information
         if ((iter < 10 || (iter % 10 == 0 && iter < 100) || (iter % 100 == 0 && iter < 1000) ||
             (iter % 1000 == 0 && iter < 10000) || (iter % 10000 == 0)) && (iter != iter_max)) {
-            Log::REDebug("GPModel parameter optimization iteration number %d", iter);
-            for (int i = 0; i < std::min((int)x.size(), 5); ++i) { Log::REDebug("(Transformed) parameter[%d]: %g", i, x[i]); }
-            if (x.size() > 5) {
-                Log::REDebug("Note: only the first 5 parameters are shown");
-            }
-            Log::REDebug("L2 norm of gradient: %g", grad_err);
-            Log::REDebug("Relative change in paramters: %g", rel_sol_change);
-            ////For debugging
-            //for (int i = 0; i < std::min((int)grad.size(), 10); ++i) { Log::REDebug("grad[%d]: %g", i, grad[i]); }
+            gradient_dummy[0] = -1.00000000001e30;//"hack" for printing nice logging information
+            gradient_dummy[1] = 1.00000000001e30;
+            opt_objfn(x, &gradient_dummy, opt_data);
         }
     }
 
